
#ifndef LUBISHEV_DB_H
#define LUBISHEV_DB_H

#include <lubishev/Types.h>

#include <QString>
#include <sqlite3.h>

namespace lubishev {

class DB {
    public:
        DB();
        ~DB();

        class Statement {
            public:
                Statement(sqlite3 *db, const QString &sql = QString());
                ~Statement();

                s32 columnAsS32(s32 columnID);
                s64 columnAsS64(s32 columnID);
                QString columnAsText(s32 columnID);
                // Throws.
                void setSQL(const QString &sql);
                // Returns true if there is anything to read. Throws.
                bool step();

            private:
                void finalize();

                sqlite3 *mDB;
                sqlite3_stmt *mStmt;
                QString mSQL;
        };

        typedef std::vector<s64> IDs;
        typedef IDs::iterator IDIt;
        
        s64 createEvent(time_t start, time_t end);
        QStringList eventTags(s64 id);
        IDs events();
        IDs eventsWithin(time_t start, time_t end);
        void eventTimes(s64 id, time_t *start, time_t *end);
        void setDir(const QString &dir);
        void setEventTags(s64 id, const QStringList &list);
        void setEventTimes(s64 id, time_t start, time_t end);

    private:
        void close();
        // Make sure the necessary tables are present.
        void ensure();
        void open(const QString &fileName);
        s64 tagID(const QString &tag);

        sqlite3 *mDB;
        Statement *mStmt;
};

} // namespace lubishev

#endif // LUBISHEV_DB_H

